import {ChangeDetectorRef, Component, Inject, Input, OnDestroy, OnInit} from '@angular/core';
import {IDataSource} from '@poweredsoft/data';
import {Subscription} from 'rxjs';

@Component({
  selector: 'bsx-ds-pagination',
  templateUrl: './data-source-pagination.component.html',
  styleUrls: ['./data-source-pagination.component.scss']
})
export class DataSourcePaginationComponent implements OnInit, OnDestroy {
  @Inject(ChangeDetectorRef) private cdf!: ChangeDetectorRef;

  @Input() dataSource: IDataSource<any>

  numberOfItems: number = 0;
  private dataSubscription: Subscription;

  ngOnDestroy(): void {
    this.dataSubscription.unsubscribe();
  }

  get pageSize() {
    return this.dataSource.pageSize;
  }

  ngOnInit(): void {
    this.dataSubscription = this.dataSource.data$.subscribe(latest => {
      if (latest)
        this.numberOfItems = latest.totalRecords;
      else
        this.numberOfItems = 0;
    });
  }
}
