import {Component, OnDestroy, OnInit} from '@angular/core';
import {IDataSource} from '@poweredsoft/data';
import {IMerchant} from 'src/app/data/services/IMerchant';
import {MerchantService} from 'src/app/data/services/merchant.service';
import {FormBuilder, Validators} from '@angular/forms';
import {Subscription} from 'rxjs';
import {IModelFormCreateEvent} from "@openharbor/ngx-data-bootstrap-ui";

@Component({
  selector: 'ps-form-group-modal-demo',
  templateUrl: './form-group-modal-demo.component.html',
  styleUrls: ['./form-group-modal-demo.component.scss']
})
export class FormGroupModalDemoComponent implements OnInit,OnDestroy {
  pages: any;

  merchantDataSource: IDataSource<IMerchant>;
  private _dataSubscription: Subscription;
  columns = ['id','name', 'address', 'actions'];

  createDataSource(): IDataSource<IMerchant> {
    return this.merchantService.createDataSource();
  }

  constructor(private merchantService: MerchantService, private fb: FormBuilder) {
    this.merchantDataSource = this.createDataSource();
  }

  ngOnDestroy(): void {
    this._dataSubscription.unsubscribe();
  }

  ngOnInit(): void {
    this.merchantDataSource.refresh();
    this._dataSubscription = this.merchantDataSource.data$.subscribe(newData => {
      if (newData)
        this.pages = new Array(newData.numberOfPages);
    });
  }

  onFormCreate(event: IModelFormCreateEvent) {
    event.shouldSetCommandModel = false;
    event.formGroup = this.fb.group({
      'name': [event.commandModel.name, Validators.required],
      'address': [event.commandModel.address, Validators.required]
    });
  }
}
